/* uio.c
   wrapper function for Win32 winsock 
 */

#include "uio-win32.h" 
#include <stdio.h>

#if 0
int		inet_aton( const char *addrstr, struct in_addr *addrptr)
{
	addrptr->S_un.S_addr = 0;	// set to initialize
	addrptr->S_un.S_addr = inet_addr(addrstr);
	if (addrptr->S_un.S_addr)
		return 1;	//valid address
	else
		return 0;	//invalid address
}
#endif

void	Win32_Sock2init(void)
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
 
	wVersionRequested = MAKEWORD( 2, 2 );
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		/* Tell the user that we could not find a usable */
		/* WinSock DLL.                                  */
		fprintf(stderr,"\n Win32 SOCK2 init failed!");
		exit(-1);
		return;
	}
 
	/* Confirm that the WinSock DLL supports 2.2.*/
	/* Note that if the DLL supports versions greater    */
	/* than 2.2 in addition to 2.2, it will still return */
	/* 2.2 in wVersion since that is the version we      */
	/* requested.                                        */
 
	if ( LOBYTE( wsaData.wVersion ) != 2 ||
			HIBYTE( wsaData.wVersion ) != 2 ) {
		/* Tell the user that we could not find a usable */
		/* WinSock DLL.                                  */
		WSACleanup( );
		fprintf( stderr,"\n Win32 SOCK2 version not supported ");
		return; 
	}
 
	/* The WinSock DLL is acceptable. Proceed. */
}
 
 
int	writev (SOCKET handle, const struct iovec iov[], int n)
{
  size_t length = 0;
  int i;
  char *buf, *ptr;
  size_t nwritten;
  
  // Determine the total length of all the buffers in <iov>.
  for (i = 0; i < n; i++)
    if ( iov[i].iov_len < 0)
      return -1;
    else
      length += iov[i].iov_len;

  buf = (char *) calloc( length, 1);
  if (!buf)
  {
  	fprintf(stderr," __writev() : Memory Alloc fail");
	return -1;
  }
  
  ptr = buf;

  for (i = 0; i < n; i++)
    {
      memcpy (ptr, iov[i].iov_base, iov[i].iov_len);
      ptr += iov[i].iov_len;
    }

  for (nwritten = 0; nwritten < length; nwritten += n)
    {
      n = send (handle, buf + nwritten,length - nwritten, 0);
      if (n == -1)
        {
          if (errno == WSAEWOULDBLOCK)
          {
            n = 0; // Keep trying to send.
          } else {
          	free(buf);
            return -1;
          }
        }
    }

  free(buf);
  return nwritten;
}


int	readv (SOCKET handle, const struct iovec iov[], int n)
{
  size_t length = 0;
  int i;
  char *buf, *ptr;

  size_t nread;

  // Determine the total length of all the buffers in <iov>.
  for (i = 0; i < n; i++)
    if ( iov[i].iov_len < 0)
      return -1;
    else
      length += iov[i].iov_len;

  buf = (char *) calloc( length, 1);
  if (!buf)
  {
  	fprintf(stderr," __readv() : Memory Alloc fail");
	return -1;
  }
  
  for (nread = 0; nread < length; nread += n)
    {
      n = recv (handle, buf + nread, length - nread, 0);
      if (n == -1)
        {
          if (errno == WSAEWOULDBLOCK)
          {
            n = 0; // Keep trying to send.
          } else {
          	free(buf);
            return -1;
          }
        }
    }


  if (length != -1)
    {
      int copyn = length;
      ptr = buf;

      for (i = 0; i < n && copyn > 0; i++)
        {
          memcpy ( iov[i].iov_base, ptr, 
          					copyn > (int)iov[i].iov_len ? iov[i].iov_len : copyn);
          ptr += iov[i].iov_len;
          copyn -= iov[i].iov_len;
        }
    }

  free(buf);
  return nread;
}

