#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <orb/orbit.h>

static void print_object_key(int len, char *buf);

static int	filesize( FILE *fp )
{
	int	set,end;
	fseek( fp, 0, SEEK_SET);
	set = ftell( fp );
	fseek( fp, 0, SEEK_END);
	end = ftell( fp );
	fseek( fp, 0, SEEK_SET);
	return (end-set);
}

int main(int argc, char **argv)
{
	CORBA_char *type_id, *host;
	CORBA_short port=0;
	CDR_Codec *codec, *encaps_codec = NULL;
	CORBA_octet *buffer, *keybuffer, endian, iiop_major, iiop_minor;
	int i, j;
	CORBA_unsigned_long len, seq_len, misclen;
	CORBA_char *str,*tmp;
	
	printf("Tags known:\n");
	printf("TAG_INTERNET_IOP: 0x%x\n", IOP_TAG_INTERNET_IOP);
	printf("TAG_MULTIPLE_COMPONENTS: 0x%x\n", IOP_TAG_MULTIPLE_COMPONENTS);
	printf("TAG_ORBIT_SPECIFIC: 0x%xl\n\n", IOP_TAG_ORBIT_SPECIFIC);

	if(argc!=2) {
		fprintf(stderr, "Usage: %s ior-string or ior-file\n", argv[0]);
		exit(-1);
	}
	
	str=argv[1];

	if(strncmp(str, "IOR:", 4)!=0) {
		fprintf(stderr, "May be Argument is IOR filename....\n or Use \n");
		fprintf(stderr, "Usage: %s ior-string\n", argv[0]);
		fprintf(stderr, "like that\n");
		goto __str_file;
		exit(-1);
	}
	
__str_file_ok:

	codec = CDR_codec_init();
	len = strlen(str);

	if((len % 2) || len <= 4) {
		fprintf(stderr, "Invalid IOR string (not even number of digits, or less than 4 digits)\n");
		exit(-1);
	}

	codec->buf_len = (len-4)/2;
	buffer = g_new(CORBA_octet, codec->buf_len);

	if(!buffer) {
		fprintf(stderr, "Memory error\n");
		exit(-1);
	}

	codec->buffer=buffer;
	codec->readonly = TRUE;

	for(j = 0, i = 4; i < len; i+=2) {
		buffer[j++] = HEXOCTET(str[i], str[i+1]);
	};

	CDR_get_octet(codec, &endian);

	codec->data_endian = endian;

	CDR_get_string(codec, &type_id);
	printf("Object ID: %s\n", type_id);

	CDR_get_seq_begin(codec, &seq_len);
	printf("Profile count: %d\n", seq_len);
	printf("\n");

	for(i = 0; i < seq_len; i++) {
		IOP_ProfileId tag;

		CDR_get_ulong(codec, &tag);
		switch(tag) {
		case IOP_TAG_INTERNET_IOP:
			printf("Profile type: TAG_INTERNET_IOP\n");

			CDR_get_ulong(codec, &misclen);
			encaps_codec = CDR_codec_init();
			encaps_codec->buffer = g_new(CORBA_octet, misclen);
			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);

			encaps_codec->buf_len = misclen;
			encaps_codec->readonly = CORBA_TRUE;
			CDR_get_octet(encaps_codec, &endian);
			encaps_codec->data_endian = endian;
			printf("Object endian: %s%s\n", endian?"Little":"Big", endian==encaps_codec->host_endian?"":" (converting)");

			CDR_get_octet(encaps_codec, &iiop_major);
			CDR_get_octet(encaps_codec, &iiop_minor);
			printf("IIOP version: %d.%d\n", iiop_major, iiop_minor);

			CDR_get_string(encaps_codec, &host);
			printf("Host: %s\n", host);

			CDR_get_ushort(encaps_codec, &port);
			printf("Port: %d\n", (unsigned short)port);

			CDR_get_seq_begin(encaps_codec, &misclen);
			keybuffer = g_new(CORBA_octet, misclen);
			CDR_buffer_gets(encaps_codec, keybuffer, misclen);
			printf("Object key: \"");
			print_object_key(misclen, keybuffer);
			printf("\"\n");

			CDR_codec_free(encaps_codec);
			encaps_codec = NULL;
			printf("\n");
			break;

		case IOP_TAG_MULTIPLE_COMPONENTS:
			/* Just skip any multiple_components data, for now */
			printf("Skipping TAG_MULTIPLE_COMPONENTS\n");
			CDR_get_ulong(codec, &misclen);
			encaps_codec = CDR_codec_init();
			encaps_codec->buf_len = misclen;
			encaps_codec->buffer = g_new(CORBA_octet, misclen);
			encaps_codec->readonly = CORBA_TRUE;
			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
			CDR_codec_free(encaps_codec); encaps_codec = NULL;
			printf("\n");
			break;

		case IOP_TAG_ORBIT_SPECIFIC:
			printf("Profile type: TAG_ORBIT_SPECIFIC\n");

			CDR_get_ulong(codec, &misclen);
			encaps_codec = CDR_codec_init();

			encaps_codec->buffer = g_new(CORBA_octet, misclen);
			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);

			encaps_codec->buf_len = misclen;
			encaps_codec->readonly = CORBA_TRUE;

			CDR_get_octet(encaps_codec, &endian);
			encaps_codec->data_endian = endian;
			printf("Object endian: %s%s\n", endian?"Little":"Big", endian==encaps_codec->host_endian?"":" (converting)");

			CDR_get_octet(encaps_codec, &iiop_major);
			CDR_get_octet(encaps_codec, &iiop_minor);
			printf("IIOP version: %d.%d\n", iiop_major, iiop_minor);

			CDR_get_string(encaps_codec, &host);
			printf("Socket path: %s\n", host);

			CDR_get_ushort(encaps_codec, &port);
			printf("IPv6 port: %d\n", port);

			CDR_get_seq_begin(encaps_codec, &misclen);
			keybuffer = g_new(CORBA_octet, misclen);
			CDR_buffer_gets(encaps_codec, keybuffer, misclen);
			printf("Object key: \"");
			print_object_key(misclen, keybuffer);
			printf("\"\n");

			CDR_codec_free(encaps_codec); encaps_codec = NULL;
			printf("\n");
			break;
		default:
			printf("Skipping unknown tag 0x%x\n", tag);

			/* Skip it */
			CDR_get_ulong(codec, &misclen);
			encaps_codec = CDR_codec_init();
			encaps_codec->buf_len = misclen;
			encaps_codec->buffer = g_new(CORBA_octet, misclen);
			encaps_codec->readonly = CORBA_TRUE;
			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
			CDR_codec_free(encaps_codec); encaps_codec = NULL;
			printf("\n");
			break;
		}
	}
	

	exit(0);
__str_file:
	tmp = (CORBA_char *)calloc(4096,1);
	{
		FILE *fp;
		int	nread,nsize;
		fp = fopen(argv[1],"rt");
		if(!fp)
		{
			fprintf(stderr,"\n\nCan't open file %s\n",argv[1]);
			exit(-2);
		}
		nsize = filesize(fp);
		nsize /= 2; nsize *= 2;
		
		nread = fread(tmp, 1,nsize,fp);
		if( strncmp("IOR:",tmp,4)!=0 ) {
			fprintf(stderr,"\n    Can't find 'IOR:' --invalid IOR file\n");
			exit(-3);
		}
		fprintf(stderr,"\n IOR filesize:%d, length = %d\n",nsize,nread);
		fclose(fp);
	}
	str = tmp;
	goto __str_file_ok;
}

static void
print_object_key(int len, char *buf)
{
	int i;
	for(i = 0; i < len; i++) {
		if(isprint(buf[i]))
			printf("%c", buf[i]);
		else
			printf(".");
	}
}

